#include "GameEngine.h"

DirectRenderer::DirectRenderer()
{
}

DirectRenderer::DirectRenderer(const DirectRenderer& orig)
{
}

DirectRenderer::~DirectRenderer()
{
    glDeleteVertexArrays(1, &vba);
    glDeleteBuffers(1, &vbo);
}

void DirectRenderer::init()
{
    glGenVertexArrays(1, &vba);
    glGenBuffers(1, &vbo);

    glBindVertexArray(vba);
    glBindBuffer(GL_ARRAY_BUFFER, vbo);

    glEnableVertexAttribArray(0);
    glVertexAttribPointer(0, 3, GL_FLOAT, false, 36, (GLvoid*) 0);

    glEnableVertexAttribArray(1);  
    glVertexAttribPointer(1, 4, GL_FLOAT, false, 36, (GLvoid*) 12);
    
    glEnableVertexAttribArray(2);  
    glVertexAttribPointer(2, 2, GL_FLOAT, false, 36, (GLvoid*) 28);
}

void DirectRenderer::prepare()
{
    int scale = GameEngine::scale;
    view.set(0, 0, (2.0f * scale) / GameEngine::width);
    view.set(1, 1, (-2.0f * scale) / GameEngine::height);
    view.set(2, 2, (-1.0f * scale) / GameEngine::height);
    
    view.set(0, 3, -1.0f);
    view.set(1, 3, 1.0f);
    view.set(2, 3, 0.5f);
    view.set(3, 3, 1.0f);

    GameEngine::updateView(view);
    glUniformMatrix4fv(GameEngine::unifProjMatrix, 1, 0, proj.getValues());
}

void DirectRenderer::drawRectangle(float minX, float minY, float maxX, float maxY,
            float tMinX, float tMinY, float tMaxX, float tMaxY,
            float r, float g, float b, float a)
{
    glBindBuffer(GL_ARRAY_BUFFER, vbo);
        
    offset += OBJECT_LENGTH;
    if(offset + OBJECT_LENGTH >= BUFFER_LENGTH)
    {
        offset = 0;
        glBufferData(GL_ARRAY_BUFFER, BUFFER_LENGTH, NULL, GL_STREAM_DRAW);
    }
    float* buffer = (float*) glMapBufferRange(GL_ARRAY_BUFFER, offset, OBJECT_LENGTH, GL_MAP_WRITE_BIT | GL_MAP_UNSYNCHRONIZED_BIT);
    if(buffer == NULL)
    {
        return;
    }

    buffer[0] = minX;
    buffer[1] = maxY;
    buffer[2] = 0.0f;
    buffer[3] = r;
    buffer[4] = g;
    buffer[5] = b;
    buffer[6] = a;
    buffer[7] = tMinX;
    buffer[8] = tMaxY;
    
    buffer[9] = maxX;
    buffer[10] = maxY;
    buffer[11] = 0.0f;
    buffer[12] = r;
    buffer[13] = g;
    buffer[14] = b;
    buffer[15] = a;
    buffer[16] = tMaxX;
    buffer[17] = tMaxY;
    
    buffer[18] = minX;
    buffer[19] = minY;
    buffer[20] = 0.0f;
    buffer[21] = r;
    buffer[22] = g;
    buffer[23] = b;
    buffer[24] = a;
    buffer[25] = tMinX;
    buffer[26] = tMinY;
    
    buffer[27] = maxX;
    buffer[28] = minY;
    buffer[29] = 0.0f;
    buffer[30] = r;
    buffer[31] = g;
    buffer[32] = b;
    buffer[33] = a;
    buffer[34] = tMaxX;
    buffer[35] = tMinY;

    glUnmapBuffer(GL_ARRAY_BUFFER);

    glBindVertexArray(vba);
    glDrawArrays(GL_TRIANGLE_STRIP, offset / (OBJECT_LENGTH / 4), 4);
}

void DirectRenderer::drawColoredRectangle(float minX, float minY, float maxX, float maxY, float r, float g, float b, float a)
{
    drawRectangle(minX, minY, maxX, maxY, 0.0f, 0.0f, 0.0f, 0.0f, r, g, b, a);
}

void DirectRenderer::drawTexturedRectangle(float minX, float minY, float maxX, float maxY, float tMinX, float tMinY, float tMaxX, float tMaxY)
{
    drawRectangle(minX, minY, maxX, maxY, tMinX, tMinY, tMaxX, tMaxY, 0.0f, 0.0f, 0.0f, 0.0f);
}